@@ -2,13 +2,30 @@
 #include "curve.h"
 #include "parse.h"
 
-static void cc_random(point_ptr p)
+static void cc_from_x(point_ptr p, element_t x)
+    //assumes there exists a point with given x coordinate
 {
-    element_t t, t1;
+    element_t t;
+    common_curve_ptr cc = p->curve->data;
+
+    element_init(t, p->curve->field);
+    p->inf_flag = 0;
+    element_mul(t, x, x);
+    element_add(t, t, cc->a);
+    element_mul(t, t, x);
+    element_add(t, t, cc->b);
+    element_sqrt(p->y, t);
+    element_set(p->x, x);
+
+    element_clear(t);
+}
+
+static void cc_random(point_ptr p)
+{
+    element_t t;
     common_curve_ptr cc = p->curve->data;
 
     element_init(t, p->curve->field);
-    element_init(t1, p->curve->field);
     p->inf_flag = 0;
     do {
 	element_random(p->x);
@@ -19,8 +36,36 @@ static void cc_random(point_ptr p)
     } while (!element_is_sqr(t));
     element_sqrt(p->y, t);
 
+    element_clear(t);
+}
+
+static void cc_from_hash(point_ptr p, int len, void *data)
+{
+    //TODO: don't find a hash by the 255th try = freeze!
+    void *datacopy;
+    element_t t, t1;
+    common_curve_ptr cc = p->curve->data;
+
+    datacopy = malloc(len);
+    memcpy(datacopy, data, len);
+
+    element_init(t, p->curve->field);
+    element_init(t1, p->curve->field);
+    p->inf_flag = 0;
+    for(;;) {
+	element_from_hash(p->x, len, datacopy);
+	element_mul(t, p->x, p->x);
+	element_add(t, t, cc->a);
+	element_mul(t, t, p->x);
+	element_add(t, t, cc->b);
+	if (element_is_sqr(t)) break;
+	((char *) datacopy)[0]++;
+    }
+    element_sqrt(p->y, t);
+
     element_clear(t);
     element_clear(t1);
+    free(datacopy);
 }
 
 static void cc_add(point_ptr r, point_ptr p, point_ptr q)
@@ -141,6 +186,8 @@ void curve_init_cc_j(curve_ptr c, element_ptr j)
 
     c->field = j->field;
     c->random = cc_random;
+    c->from_x = cc_from_x;
+    c->from_hash = cc_from_hash;
     c->add = cc_add;
     c->mul = cc_mul;
     c->data = malloc(sizeof(common_curve_t));
@@ -164,6 +211,8 @@ void curve_init_cc_ab(curve_ptr c, element_ptr a, element_ptr b)
     common_curve_ptr cc;
     c->field = a->field;
     c->random = cc_random;
+    c->from_x = cc_from_x;
+    c->from_hash = cc_from_hash;
     c->add = cc_add;
     c->mul = cc_mul;
     c->data = malloc(sizeof(common_curve_t));
