@@ -17,6 +17,8 @@ typedef struct point_s point_t[1];
 struct curve_s {
     field_ptr field;
     void (*random)(point_ptr);
+    void (*from_x)(point_ptr, element_ptr);
+    void (*from_hash)(point_ptr, int, void *);
     void (*add)(point_ptr, point_ptr, point_ptr);
     void (*mul)(point_ptr, mpz_ptr, point_ptr);
     void (*curve_clear)(struct curve_s *curve);
@@ -50,6 +52,11 @@ static inline void point_random(point_ptr p)
     p->curve->random(p);
 }
 
+static inline void point_from_hash(point_ptr p, int len, void *data)
+{
+    p->curve->from_hash(p, len, data);
+}
+
 static inline void point_add(point_ptr r, point_ptr p, point_ptr q)
 {
     r->curve->add(r, p, q);
@@ -66,15 +73,15 @@ static inline void point_set(point_ptr p, point_ptr q)
     element_set(p->y, q->y);
 }
 
-static inline void point_out_str(FILE *stream, point_ptr p)
+static inline void point_out_str(FILE *stream, int base, point_ptr p)
 {
     if (p->inf_flag) {
 	fputc('O', stream);
 	return;
     }
-    element_out_str(stream, p->x);
+    element_out_str(stream, base, p->x);
     fputc(' ', stream);
-    element_out_str(stream, p->y);
+    element_out_str(stream, base, p->y);
 }
 
 static inline void point_mul(point_ptr r, mpz_ptr n, point_ptr p)
@@ -92,6 +99,11 @@ static inline int point_is_inf(point_ptr p)
     return p->inf_flag;
 }
 
+static inline void point_from_x(point_ptr p, element_ptr x)
+{
+    p->curve->from_x(p, x);
+}
+
 //I define "common" curves (abbr. "cc") to be those
 //of the form y^2 = x^3 + ax + b defined over a field with
 //characteristic > 3
