@@ -30,7 +30,7 @@ struct field_s {
     void (*set)(element_ptr, element_ptr);
     void (*set0)(element_ptr);
     void (*set1)(element_ptr);
-    void (*out_str)(FILE *stream, element_ptr);
+    void (*out_str)(FILE *stream, int base, element_ptr);
     void (*add)(element_ptr, element_ptr, element_ptr);
     void (*sub)(element_ptr, element_ptr, element_ptr);
     void (*mul)(element_ptr, element_ptr, element_ptr);
@@ -38,12 +38,17 @@ struct field_s {
     void (*neg)(element_ptr, element_ptr);
     int (*cmp)(element_ptr, element_ptr);
     void (*random)(element_ptr);
+    void (*from_hash)(element_ptr, int len, void *data);
     int (*is1)(element_ptr);
     int (*is0)(element_ptr);
     int (*is_sqr)(element_ptr);
     void (*sqrt)(element_ptr, element_ptr);
+    int (*to_bytes)(unsigned char *data, element_ptr);
+    int (*from_bytes)(element_ptr, unsigned char *data);
+    int (*length_in_bytes)(element_ptr);
+    int fixed_length_in_bytes; //length of an element in bytes; -1 for variable
     element_t nqr; //nonquadratic residue
-    mpz_t order; //0 for infinite order
+    mpz_t order; //-1 for infinite order
     void *data;
 };
 typedef struct field_s *field_ptr;
@@ -73,9 +78,9 @@ static inline void element_clear(element_ptr e)
     e->field = NULL;
 }
 
-static inline void element_out_str(FILE *stream, element_ptr e)
+static inline void element_out_str(FILE *stream, int base, element_ptr e)
 {
-    e->field->out_str(stream, e);
+    e->field->out_str(stream, base, e);
 }
 
 static inline void element_set_si(element_ptr e, signed long int i)
@@ -158,6 +163,30 @@ static inline void element_sqrt(element_ptr a, element_ptr b)
     a->field->sqrt(a, b);
 }
 
+static inline void element_from_hash(element_ptr a, int len, void *data)
+{
+    a->field->from_hash(a, len, data);
+}
+
+static inline int element_to_bytes(unsigned char *data, element_ptr a)
+{
+    return a->field->to_bytes(data, a);
+}
+
+static inline int element_from_bytes(element_ptr a, unsigned char *data)
+{
+    return a->field->from_bytes(a, data);
+}
+
+static inline int element_length_in_bytes(element_ptr a)
+{
+    if (a->field->fixed_length_in_bytes < 0) {
+	return a->field->length_in_bytes(a);
+    } else {
+	return a->field->fixed_length_in_bytes;
+    }
+}
+
 void field_init_fp(field_ptr f, mpz_t prime);
 void element_pow(element_ptr x, element_ptr a, mpz_ptr n);
 #endif //FIELD_H
