@@ -46,9 +46,9 @@ static void zp_set1(element_ptr e)
     mpz_set_si(e->data, 1);
 }
 
-static void zp_out_str(FILE *stream, element_ptr e)
+static void zp_out_str(FILE *stream, int base, element_ptr e)
 {
-    mpz_out_str(stream, 10, e->data);
+    mpz_out_str(stream, base, e->data);
 }
 
 static void zp_add(element_ptr n, element_ptr a, element_ptr b)
@@ -89,6 +89,17 @@ static void zp_random(element_ptr n)
     pbc_mpz_random(n->data, n->field->order);
 }
 
+static void zp_from_hash(element_ptr n, int len, void *data)
+    //TODO: something more sophisticated!
+{
+    mpz_t z;
+
+    mpz_init(z);
+    mpz_import(z, len, 1, 1, 0, 0, data);
+    zp_set_mpz(n, z);
+    mpz_clear(z);
+}
+
 static int zp_is1(element_ptr n)
 {
     return !mpz_cmp_ui((mpz_ptr) n->data, 1);
@@ -161,6 +172,47 @@ static void zp_field_clear(field_t f)
     free(f->nqr->data);
 }
 
+static int zp_to_bytes(unsigned char *data, element_t e)
+{
+    mpz_t z;
+    int i, n;
+    unsigned char *ptr;
+
+    mpz_init(z);
+    mpz_set(z, e->data);
+    n = e->field->fixed_length_in_bytes;
+    ptr = data;
+    for (i = 0; i < n; i++) {
+	*ptr = (unsigned char) mpz_get_ui(z);
+	ptr++;
+	mpz_tdiv_q_2exp(z, z, 8);
+    }
+    mpz_clear(z);
+    return n;
+}
+
+static int zp_from_bytes(element_t e, unsigned char *data)
+{
+    unsigned char *ptr;
+    int i, n;
+    mpz_ptr z = e->data;
+    mpz_t z1;
+
+    mpz_init(z1);
+    mpz_set_ui(z, 0);
+
+    ptr = data;
+    n = e->field->fixed_length_in_bytes;
+    for (i=0; i<n; i++) {
+	mpz_set_ui(z1, *ptr);
+	mpz_mul_2exp(z1, z1, i * 8);
+	ptr++;
+	mpz_add(z, z, z1);
+    }
+    mpz_clear(z1);
+    return n;
+}
+
 void field_init_fp(field_ptr f, mpz_t prime)
 {
     f->init = zp_init;
@@ -176,6 +228,7 @@ void field_init_fp(field_ptr f, mpz_t prime)
     f->cmp = zp_cmp;
     f->invert = zp_invert;
     f->random = zp_random;
+    f->from_hash = zp_from_hash;
     f->is1 = zp_is1;
     f->is0 = zp_is0;
     f->set0 = zp_set0;
@@ -183,6 +236,8 @@ void field_init_fp(field_ptr f, mpz_t prime)
     f->is_sqr = zp_is_sqr;
     f->sqrt = fp_tonelli;
     f->field_clear = zp_field_clear;
+    f->to_bytes = zp_to_bytes;
+    f->from_bytes = zp_from_bytes;
 
     mpz_init(f->order);
     mpz_set(f->order, prime);
@@ -193,6 +248,7 @@ void field_init_fp(field_ptr f, mpz_t prime)
 	pbc_mpz_random(f->nqr->data, prime);
 	if (mpz_legendre(f->nqr->data, prime) == -1) break;
     }
+    f->fixed_length_in_bytes = (mpz_sizeinbase(prime, 2) + 7) / 8;
 }
 
 void element_pow(element_ptr x, element_ptr a, mpz_ptr n)
