@@ -1,12 +1,5 @@
 #include "pairing.h"
 
-struct curve_group_data_s {
-    curve_ptr curve;
-    mpz_t cofac;
-};
-typedef struct curve_group_data_s curve_group_data_t[1];
-typedef struct curve_group_data_s *curve_group_data_ptr;
-
 static void curve_group_init(element_ptr e)
 {
     e->data = malloc(sizeof(point_t));
@@ -41,9 +34,22 @@ static void curve_group_random(element_ptr x)
     point_mul(x->data, p->cofac, x->data);
 }
 
-static void curve_group_out_str(FILE *stream, element_ptr x)
+static void curve_group_from_hash(element_ptr x, int len, void *data)
 {
-    point_out_str(stream, x->data);
+    curve_group_data_ptr p = x->field->data;
+    point_from_hash(x->data, len, data);
+    point_mul(x->data, p->cofac, x->data);
+}
+
+static void curve_group_out_str(FILE *stream, int base, element_ptr x)
+{
+    point_out_str(stream, base, x->data);
+}
+
+static int curve_group_length_in_bytes(element_ptr x)
+{
+    point_ptr P = x->data;
+    return element_length_in_bytes(P->x) + element_length_in_bytes(P->y);
 }
 
 void field_clear_curve_group(field_t f)
@@ -67,8 +73,14 @@ void field_init_curve_group(field_t f, curve_t c, mpz_t cofac)
     f->set1 = curve_group_set1;
     f->set = curve_group_set;
     f->random = curve_group_random;
+    f->from_hash = curve_group_from_hash;
     f->out_str = curve_group_out_str;
     f->field_clear = field_clear_curve_group;
+    if (c->field->fixed_length_in_bytes < 0) {
+	f->length_in_bytes = curve_group_length_in_bytes;
+    } else {
+	f->fixed_length_in_bytes = 2 * c->field->fixed_length_in_bytes;
+    }
 }
 
 static void cc_pairing(element_ptr out, element_ptr in1, element_ptr in2,
@@ -79,6 +91,39 @@ static void cc_pairing(element_ptr out, element_ptr in1, element_ptr in2,
     element_pow(out, out, p->tateexp);
 }
 
+static int cc_is_almost_coddh(element_ptr a, element_ptr b,
+	element_ptr c, element_ptr d,
+	pairing_t pairing)
+{
+    int res = 0;
+    element_t t0, t1, t2;
+
+    element_init(t0, pairing->GT);
+    element_init(t1, pairing->GT);
+    element_init(t2, pairing->GT);
+    mnt_pairing_data_ptr p = pairing->data;
+    cc_miller(t0, pairing->r, a->data, d->data);
+    cc_miller(t1, pairing->r, b->data, c->data);
+    element_pow(t0, t0, p->tateexp);
+    element_pow(t1, t1, p->tateexp);
+    element_mul(t2, t0, t1);
+    if (element_is1(t2)) {
+	//g, g^x, h, h^-x case
+	res = 1;
+    } else {
+	element_invert(t1, t1);
+	element_mul(t2, t0, t1);
+	if (element_is1(t2)) {
+	    //g, g^x, h, h^x case
+	    res = 1;
+	}
+    }
+    element_clear(t0);
+    element_clear(t1);
+    element_clear(t2);
+    return res;
+}
+
 void pairing_init_cc_param(pairing_t pairing, cc_param_t param)
 {
     mnt_pairing_data_ptr p;
@@ -88,7 +133,9 @@ void pairing_init_cc_param(pairing_t pairing, cc_param_t param)
 
     mpz_init(pairing->r);
     mpz_set(pairing->r, param->r);
+    field_init_fp(pairing->Zr, pairing->r);
     pairing->map = cc_pairing;
+    pairing->is_almost_coddh = cc_is_almost_coddh;
 
     p =	pairing->data = malloc(sizeof(mnt_pairing_data_t));
     field_init_fp(p->Fq, param->q);
