@@ -253,6 +253,18 @@ static void polymod_random(element_ptr f)
     poly_remove_leading_zeroes(f);
 }
 
+static void polymod_from_hash(element_ptr f, int len, void *data)
+{
+    polymod_field_data_ptr p = f->field->data;
+    int n = poly_degree(p->poly);
+    int i;
+    poly_alloc(f, n);
+    for (i=0; i<n; i++) {
+	element_from_hash(poly_coeff(f, i), len, data);
+    }
+    poly_remove_leading_zeroes(f);
+}
+
 static void polymod_mul(element_ptr res, element_ptr f, element_ptr g)
 {
     element_t prod;
@@ -308,7 +320,7 @@ static int poly_is1(element_ptr e)
     return 0;
 }
 
-static void poly_out_str(FILE *stream, element_ptr e)
+static void poly_out_str(FILE *stream, int base, element_ptr e)
 {
     int i;
     int n = poly_coeff_count(e);
@@ -320,7 +332,7 @@ static void poly_out_str(FILE *stream, element_ptr e)
     fputc('[', stream);
     for (i=0; i<n; i++) {
 	if (i) fputc(' ', stream);
-	element_out_str(stream, poly_coeff(e, i));
+	element_out_str(stream, 0, poly_coeff(e, i));
     }
     fputc(']', stream);
 }
@@ -608,6 +620,47 @@ void poly_make_monic(element_t f, element_t g)
     element_set1(e0);
 }
 
+static int polymod_to_bytes(unsigned char *data, element_t f)
+{
+    polymod_field_data_ptr p = f->field->data;
+    int n = poly_degree(p->poly);
+    int i;
+    int len = 0;
+
+    for (i=0; i<n; i++) {
+	element_ptr e = poly_coeff(f, i);
+	len += element_to_bytes(data + len, e);
+    }
+    return len;
+}
+
+static int polymod_length_in_bytes(element_t f)
+{
+    polymod_field_data_ptr p = f->field->data;
+    int n = poly_degree(p->poly);
+    int res = 0;
+    int i;
+
+    for (i=0; i<n; i++) {
+	res += element_length_in_bytes(poly_coeff(f, i));
+    }
+
+    return res;
+}
+
+static int polymod_from_bytes(element_t f, unsigned char *data)
+{
+    polymod_field_data_ptr p = f->field->data;
+    int n = poly_degree(p->poly);
+    int len = 0;
+    int i;
+
+    for (i=0; i<n; i++) {
+	len += element_from_bytes(poly_coeff(f, i), data + len);
+    }
+    return len;
+}
+
 void compute_x_powers(darray_ptr xpwr, element_ptr poly)
 //compute x^n,...,x^{2n-2} mod poly
 {
@@ -645,7 +698,7 @@ void compute_x_powers(darray_ptr xpwr, element_ptr poly)
 }
 
 void field_init_polymod_no_nqr(field_ptr f, element_ptr poly)
-    //assumes f is monic
+    //assumes poly is monic
 {
     poly_field_data_ptr pdp = poly->field->data;
     poly_element_ptr pd = poly->data;
@@ -677,9 +730,18 @@ void field_init_polymod_no_nqr(field_ptr f, element_ptr poly)
     f->cmp = poly_cmp;
     f->mul = polymod_mul;
     f->random = polymod_random;
+    f->from_hash = polymod_from_hash;
     f->invert = polymod_invert;
     f->is_sqr = polymod_is_sqr;
     f->sqrt = polymod_sqrt;
+    f->to_bytes = polymod_to_bytes;
+    f->from_bytes = polymod_from_bytes;
+    if (poly->field->fixed_length_in_bytes < 0) {
+	f->fixed_length_in_bytes = -1;
+	f->length_in_bytes = polymod_length_in_bytes;
+    } else {
+	f->fixed_length_in_bytes = poly->field->fixed_length_in_bytes * poly_degree(poly);
+    }
     mpz_init(f->order);
     n = pd->coeff->count;
     mpz_pow_ui(f->order, p->field->order, n - 1);
